@@ -1,23 +1,30 @@
-import traceback
 from functools import partial
 import re
+import os
+import sys
 
 from .flowspec import FlowSpec
-from .exception import MetaflowInternalError, \
-    MetaflowException, InvalidDecoratorAttribute
+from .exception import (
+    MetaflowInternalError,
+    MetaflowException,
+    InvalidDecoratorAttribute,
+)
 
 import click
 
+
 class BadStepDecoratorException(MetaflowException):
     headline = "Syntax error"
 
     def __init__(self, deco, func):
-        msg =\
-            "You tried to apply decorator '{deco}' on '{func}' which is "\
-            "not declared as a @step. Make sure you apply this decorator "\
-            "on a function which has @step on the line just before the "\
-            "function name and @{deco} is above @step.".format(deco=deco,
-                                                               func=func.__name__)
+        msg = (
+            "You tried to apply decorator '{deco}' on '{func}' which is "
+            "not declared as a @step. Make sure you apply this decorator "
+            "on a function which has @step on the line just before the "
+            "function name and @{deco} is above @step.".format(
+                deco=deco, func=func.__name__
+            )
+        )
         super(BadStepDecoratorException, self).__init__(msg)
 
 
@@ -25,9 +32,10 @@ class BadFlowDecoratorException(MetaflowException):
     headline = "Syntax error"
 
     def __init__(self, deconame):
-        msg =\
-            "Decorator '%s' can be applied only to FlowSpecs. Make sure "\
+        msg = (
+            "Decorator '%s' can be applied only to FlowSpecs. Make sure "
             "the decorator is above a class definition." % deconame
+        )
         super(BadFlowDecoratorException, self).__init__(msg)
 
 
@@ -36,10 +44,14 @@ class UnknownStepDecoratorException(MetaflowException):
 
     def __init__(self, deconame):
         from .plugins import STEP_DECORATORS
-        decos = ', '.join(t.name for t in STEP_DECORATORS 
-                            if not t.name.endswith('_internal'))
-        msg = "Unknown step decorator *{deconame}*. The following decorators are "\
-              "supported: *{decos}*".format(deconame=deconame, decos=decos)
+
+        decos = ", ".join(
+            t.name for t in STEP_DECORATORS if not t.name.endswith("_internal")
+        )
+        msg = (
+            "Unknown step decorator *{deconame}*. The following decorators are "
+            "supported: *{decos}*".format(deconame=deconame, decos=decos)
+        )
         super(UnknownStepDecoratorException, self).__init__(msg)
 
 
@@ -47,9 +59,12 @@ class DuplicateStepDecoratorException(MetaflowException):
     headline = "Duplicate decorators"
 
     def __init__(self, deco, func):
-        msg = "Step '{step}' already has a decorator '{deco}'. "\
-              "You can specify each decorator only once."\
-              .format(step=func.__name__, deco=deco)
+        msg = (
+            "Step '{step}' already has a decorator '{deco}'. "
+            "You can specify each decorator only once.".format(
+                step=func.__name__, deco=deco
+            )
+        )
         super(DuplicateStepDecoratorException, self).__init__(msg)
 
 
@@ -58,9 +73,12 @@ class UnknownFlowDecoratorException(MetaflowException):
 
     def __init__(self, deconame):
         from .plugins import FLOW_DECORATORS
-        decos = ', '.join(t.name for t in FLOW_DECORATORS)
-        msg = "Unknown flow decorator *{deconame}*. The following decorators are "\
-              "supported: *{decos}*".format(deconame=deconame, decos=decos)
+
+        decos = ", ".join(t.name for t in FLOW_DECORATORS)
+        msg = (
+            "Unknown flow decorator *{deconame}*. The following decorators are "
+            "supported: *{decos}*".format(deconame=deconame, decos=decos)
+        )
         super(UnknownFlowDecoratorException, self).__init__(msg)
 
 
@@ -68,9 +86,10 @@ class DuplicateFlowDecoratorException(MetaflowException):
     headline = "Duplicate decorators"
 
     def __init__(self, deco):
-        msg = "Flow already has a decorator '{deco}'. "\
-              "You can specify each decorator only once."\
-              .format(deco=deco)
+        msg = (
+            "Flow already has a decorator '{deco}'. "
+            "You can specify each decorator only once.".format(deco=deco)
+        )
         super(DuplicateFlowDecoratorException, self).__init__(msg)
 
 
@@ -79,12 +98,10 @@ class Decorator(object):
     Base class for all decorators.
     """
 
-    name = 'NONAME'
+    name = "NONAME"
     defaults = {}
 
-    def __init__(self,
-                 attributes=None,
-                 statically_defined=False):
+    def __init__(self, attributes=None, statically_defined=False):
         self.attributes = self.defaults.copy()
         self.statically_defined = statically_defined
 
@@ -93,34 +110,35 @@ class Decorator(object):
                 if k in self.defaults:
                     self.attributes[k] = v
                 else:
-                    raise InvalidDecoratorAttribute(
-                        self.name, k, self.defaults)
+                    raise InvalidDecoratorAttribute(self.name, k, self.defaults)
 
     @classmethod
     def _parse_decorator_spec(cls, deco_spec):
-        top = deco_spec.split(':', 1)
+        top = deco_spec.split(":", 1)
         if len(top) == 1:
             return cls()
         else:
             name, attrspec = top
-            attrs = dict(map(lambda x: x.strip(), a.split('=')) 
-                for a in re.split(''',(?=[\s\w]+=)''', attrspec.strip('"\'')))
+            attrs = dict(
+                map(lambda x: x.strip(), a.split("="))
+                for a in re.split(""",(?=[\s\w]+=)""", attrspec.strip("\"'"))
+            )
             return cls(attributes=attrs)
 
     def make_decorator_spec(self):
         attrs = {k: v for k, v in self.attributes.items() if v is not None}
         if attrs:
-            attrstr = ','.join('%s=%s' % x for x in attrs.items())
-            return '%s:%s' % (self.name, attrstr)
+            attrstr = ",".join("%s=%s" % x for x in attrs.items())
+            return "%s:%s" % (self.name, attrstr)
         else:
             return self.name
 
     def __str__(self):
-        mode = 'decorated' if self.statically_defined else 'cli'
-        attrs = ' '.join('%s=%s' % x for x in self.attributes.items())
+        mode = "decorated" if self.statically_defined else "cli"
+        attrs = " ".join("%s=%s" % x for x in self.attributes.items())
         if attrs:
-            attrs = ' ' + attrs
-        fmt = '%s<%s%s>' % (self.name, mode, attrs)
+            attrs = " " + attrs
+        fmt = "%s<%s%s>" % (self.name, mode, attrs)
         return fmt
 
 
@@ -135,7 +153,9 @@ class FlowDecorator(Decorator):
         self._flow_decorators.append(self)
         super(FlowDecorator, self).__init__(*args, **kwargs)
 
-    def flow_init(self, flow, graph,  environment, datastore, logger, echo, options):
+    def flow_init(
+        self, flow, graph, environment, flow_datastore, metadata, logger, echo, options
+    ):
         """
         Called when all decorators have been created for this flow.
         """
@@ -159,20 +179,23 @@ def add_decorator_options(cmd):
     for deco in flow_decorators():
         for option, kwargs in deco.options.items():
             if option in seen:
-                msg = "Flow decorator '%s' uses an option '%s' which is also "\
-                      "used by the decorator '%s'. This is a bug in Metaflow. "\
-                      "Please file a ticket on GitHub."\
-                      % (deco.name, option, seen[option])
+                msg = (
+                    "Flow decorator '%s' uses an option '%s' which is also "
+                    "used by the decorator '%s'. This is a bug in Metaflow. "
+                    "Please file a ticket on GitHub."
+                    % (deco.name, option, seen[option])
+                )
                 raise MetaflowInternalError(msg)
             else:
                 seen[option] = deco.name
-                cmd.params.insert(0, click.Option(('--' + option,), **kwargs))
+                cmd.params.insert(0, click.Option(("--" + option,), **kwargs))
     return cmd
 
 
 def flow_decorators():
     return FlowDecorator._flow_decorators
 
+
 class StepDecorator(Decorator):
     """
     Base class for all step decorators.
@@ -199,11 +222,13 @@ class StepDecorator(Decorator):
     state easily.
 
     TODO (savin): Initialize the decorators with flow, graph,
-                  step.__name__ etc., so that we don't have to 
-                  pass them around with every lifecycle call. 
+                  step.__name__ etc., so that we don't have to
+                  pass them around with every lifecycle call.
     """
 
-    def step_init(self, flow, graph, step_name, decorators, environment, datastore, logger):
+    def step_init(
+        self, flow, graph, step_name, decorators, environment, flow_datastore, logger
+    ):
         """
         Called when all decorators have been created for this step
         """
@@ -215,6 +240,20 @@ class StepDecorator(Decorator):
         """
         pass
 
+    def add_to_package(self):
+        """
+        Called to add custom packages needed for a decorator. This hook will be
+        called in the `MetaflowPackage` class where metaflow compiles the code package
+        tarball. This hook is invoked in the `MetaflowPackage`'s `path_tuples`
+        function. The `path_tuples` function is a generator that yields a tuple of
+        `(file_path, arcname)`.`file_path` is the path of the file in the local file system;
+        the `arcname` is the path of the file in the constructed tarball or the path of the file
+        after decompressing the tarball.
+
+        Returns a list of tuples where each tuple represents (file_path, arcname)
+        """
+        return []
+
     def step_task_retry_count(self):
         """
         Called to determine the number of times this task should be retried.
@@ -222,8 +261,8 @@ class StepDecorator(Decorator):
         are attempts to run the process after the user code has failed all
         its retries.
 
-        Typically, the runtime takes the maximum of retry counts across 
-        decorators and user specification to determine the task retry count. 
+        Typically, the runtime takes the maximum of retry counts across
+        decorators and user specification to determine the task retry count.
         If you want to force no retries, return the special values (None, None).
         """
         return 0, 0
@@ -235,13 +274,9 @@ class StepDecorator(Decorator):
         """
         pass
 
-    def runtime_task_created(self,
-                             datastore,
-                             task_id,
-                             split_index,
-                             input_paths,
-                             is_cloned,
-                             ubf_context):
+    def runtime_task_created(
+        self, task_datastore, task_id, split_index, input_paths, is_cloned, ubf_context
+    ):
         """
         Called when the runtime has created a task related to this step.
         """
@@ -253,61 +288,50 @@ class StepDecorator(Decorator):
         """
         pass
 
-    def runtime_step_cli(self,
-                         cli_args,
-                         retry_count,
-                         max_user_code_retries,
-                         ubf_context):
+    def runtime_step_cli(
+        self, cli_args, retry_count, max_user_code_retries, ubf_context
+    ):
         """
         Access the command line for a step execution in the runtime context.
         """
         pass
 
-    def task_pre_step(self,
-                      step_name,
-                      datastore,
-                      metadata,
-                      run_id,
-                      task_id,
-                      flow,
-                      graph,
-                      retry_count,
-                      max_user_code_retries,
-                      ubf_context,
-                      inputs):
+    def task_pre_step(
+        self,
+        step_name,
+        task_datastore,
+        metadata,
+        run_id,
+        task_id,
+        flow,
+        graph,
+        retry_count,
+        max_user_code_retries,
+        ubf_context,
+        inputs,
+    ):
         """
         Run before the step function in the task context.
         """
         pass
 
-    def task_decorate(self,
-                      step_func,
-                      flow,
-                      graph,
-                      retry_count,
-                      max_user_code_retries,
-                      ubf_context):
+    def task_decorate(
+        self, step_func, flow, graph, retry_count, max_user_code_retries, ubf_context
+    ):
         return step_func
 
-    def task_post_step(self,
-                       step_name,
-                       flow,
-                       graph,
-                       retry_count,
-                       max_user_code_retries):
+    def task_post_step(
+        self, step_name, flow, graph, retry_count, max_user_code_retries
+    ):
         """
         Run after the step function has finished successfully in the task
         context.
         """
         pass
 
-    def task_exception(self,
-                       exception,
-                       step_name,
-                       flow,
-                       graph,
-                       retry_count,
-                       max_user_code_retries):
+    def task_exception(
+        self, exception, step_name, flow, graph, retry_count, max_user_code_retries
+    ):
         """
         Run if the step function raised an exception in the task context.
 
@@ -316,13 +340,9 @@ class StepDecorator(Decorator):
         """
         pass
 
-    def task_finished(self,
-                      step_name,
-                      flow,
-                      graph,
-                      is_task_ok,
-                      retry_count,
-                      max_user_code_retries):
+    def task_finished(
+        self, step_name, flow, graph, is_task_ok, retry_count, max_user_code_retries
+    ):
         """
         Run after the task context has been finalized.
 
@@ -354,8 +374,9 @@ def _base_flow_decorator(decofunc, *args, **kwargs):
             if decofunc.name in cls._flow_decorators:
                 raise DuplicateFlowDecoratorException(decofunc.name)
             else:
-                cls._flow_decorators[decofunc.name] = decofunc(attributes=kwargs,
-                                                               statically_defined=True)
+                cls._flow_decorators[decofunc.name] = decofunc(
+                    attributes=kwargs, statically_defined=True
+                )
         else:
             raise BadFlowDecoratorException(decofunc.name)
         return cls
@@ -365,6 +386,7 @@ def _base_flow_decorator(decofunc, *args, **kwargs):
         # function to be decorated as the first argument.
         def wrap(f):
             return _base_flow_decorator(decofunc, f, **kwargs)
+
         return wrap
 
 
@@ -377,15 +399,14 @@ def _base_step_decorator(decotype, *args, **kwargs):
         # No keyword arguments specified for the decorator, e.g. @foobar.
         # The first argument is the function to be decorated.
         func = args[0]
-        if not hasattr(func, 'is_step'):
+        if not hasattr(func, "is_step"):
             raise BadStepDecoratorException(decotype.name, func)
 
         # Only the first decorator applies
         if decotype.name in [deco.name for deco in func.decorators]:
             raise DuplicateStepDecoratorException(decotype.name, func)
         else:
-            func.decorators.append(decotype(attributes=kwargs,
-                                            statically_defined=True))
+            func.decorators.append(decotype(attributes=kwargs, statically_defined=True))
 
         return func
     else:
@@ -394,6 +415,7 @@ def _base_step_decorator(decotype, *args, **kwargs):
         # function to be decorated as the first argument.
         def wrap(f):
             return _base_step_decorator(decotype, f, **kwargs)
+
         return wrap
 
 
@@ -412,6 +434,7 @@ def _attach_decorators(flow, decospecs):
     for step in flow:
         _attach_decorators_to_step(step, decospecs)
 
+
 def _attach_decorators_to_step(step, decospecs):
     """
     Attach decorators to a step during runtime. This has the same
@@ -419,9 +442,10 @@ def _attach_decorators_to_step(step, decospecs):
     the step.
     """
     from .plugins import STEP_DECORATORS
+
     decos = {decotype.name: decotype for decotype in STEP_DECORATORS}
     for decospec in decospecs:
-        deconame = decospec.strip("'").split(':')[0]
+        deconame = decospec.strip("'").split(":")[0]
         if deconame not in decos:
             raise UnknownStepDecoratorException(deconame)
         # Attach the decorator to step if it doesn't have the decorator
@@ -431,17 +455,29 @@ def _attach_decorators_to_step(step, decospecs):
             deco = decos[deconame]._parse_decorator_spec(decospec)
             step.decorators.append(deco)
 
-def _init_flow_decorators(flow, graph, environment, datastore, logger, echo, deco_options):
+
+def _init_flow_decorators(
+    flow, graph, environment, flow_datastore, metadata, logger, echo, deco_options
+):
     for deco in flow._flow_decorators.values():
         opts = {option: deco_options[option] for option in deco.options}
-        deco.flow_init(flow, graph, environment, datastore, logger, echo, opts)
+        deco.flow_init(
+            flow, graph, environment, flow_datastore, metadata, logger, echo, opts
+        )
 
 
-def _init_step_decorators(flow, graph, environment, datastore, logger):
+def _init_step_decorators(flow, graph, environment, flow_datastore, logger):
     for step in flow:
         for deco in step.decorators:
-            deco.step_init(flow, graph, step.__name__,
-                           step.decorators, environment, datastore, logger)
+            deco.step_init(
+                flow,
+                graph,
+                step.__name__,
+                step.decorators,
+                environment,
+                flow_datastore,
+                logger,
+            )
 
 
 def step(f):
@@ -461,10 +497,11 @@ def step(f):
 
 def _import_plugin_decorators(globals_dict):
     """
-    Auto-generate a decorator function for every decorator 
+    Auto-generate a decorator function for every decorator
     defined in plugins.STEP_DECORATORS and plugins.FLOW_DECORATORS.
     """
     from .plugins import STEP_DECORATORS, FLOW_DECORATORS
+
     # Q: Why not use StepDecorators directly as decorators?
     # A: Getting an object behave as a decorator that can work
     #    both with and without arguments is surprisingly hard.
