@@ -4,26 +4,26 @@ from collections import namedtuple
 import click
 
 from .util import get_username, is_stringish
-from .exception import ParameterFieldFailed,\
-                       ParameterFieldTypeMismatch,\
-                       MetaflowException
+from .exception import (
+    ParameterFieldFailed,
+    ParameterFieldTypeMismatch,
+    MetaflowException,
+)
 
 try:
     # Python2
     strtype = basestring
-except:
+except NameError:
     # Python3
     strtype = str
 
 # ParameterContext allows deploy-time functions modify their
 # behavior based on the context. We can add fields here without
 # breaking backwards compatibility but don't remove any fields!
-ParameterContext = namedtuple('ParameterContext',
-                              ['flow_name',
-                               'user_name',
-                               'parameter_name',
-                               'logger',
-                               'ds_type'])
+ParameterContext = namedtuple(
+    "ParameterContext",
+    ["flow_name", "user_name", "parameter_name", "logger", "ds_type"],
+)
 
 # currently we execute only one flow per process, so we can treat
 # Parameters globally. If this was to change, it should/might be
@@ -32,8 +32,9 @@ ParameterContext = namedtuple('ParameterContext',
 parameters = []
 context_proto = None
 
+
 class JSONTypeClass(click.ParamType):
-    name = 'JSON'
+    name = "JSON"
 
     def convert(self, value, param, ctx):
         if not isinstance(value, strtype):
@@ -48,7 +49,8 @@ class JSONTypeClass(click.ParamType):
         return repr(self)
 
     def __repr__(self):
-        return 'JSON'
+        return "JSON"
+
 
 class DeployTimeField(object):
     """
@@ -59,20 +61,25 @@ class DeployTimeField(object):
     object curries the context argument for the function, and pretty
     prints any exceptions that occur during evaluation.
     """
-    def __init__(self,
-                 parameter_name,
-                 parameter_type,
-                 field,
-                 fun,
-                 return_str=True,
-                 print_representation=None):
+
+    def __init__(
+        self,
+        parameter_name,
+        parameter_type,
+        field,
+        fun,
+        return_str=True,
+        print_representation=None,
+    ):
 
         self.fun = fun
         self.field = field
         self.parameter_name = parameter_name
         self.parameter_type = parameter_type
         self.return_str = return_str
-        self.print_representation = self.user_print_representation = print_representation
+        self.print_representation = (
+            self.user_print_representation
+        ) = print_representation
         if self.print_representation is None:
             self.print_representation = str(self.fun)
 
@@ -99,23 +106,22 @@ class DeployTimeField(object):
 
         # note: this doesn't work with long in Python2 or types defined as
         # click types, e.g. click.INT
-        TYPES = {bool: 'bool',
-                 int: 'int',
-                 float: 'float',
-                 list: 'list'}
+        TYPES = {bool: "bool", int: "int", float: "float", list: "list"}
 
-        msg = "The value returned by the deploy-time function for "\
-              "the parameter *%s* field *%s* has a wrong type. " %\
-              (self.parameter_name, self.field)
+        msg = (
+            "The value returned by the deploy-time function for "
+            "the parameter *%s* field *%s* has a wrong type. "
+            % (self.parameter_name, self.field)
+        )
 
         if self.parameter_type in TYPES:
             if type(val) != self.parameter_type:
-                msg += 'Expected a %s.' % TYPES[self.parameter_type]
+                msg += "Expected a %s." % TYPES[self.parameter_type]
                 raise ParameterFieldTypeMismatch(msg)
             return str(val) if self.return_str else val
         else:
             if not is_stringish(val):
-                msg += 'Expected a string.'
+                msg += "Expected a string."
                 raise ParameterFieldTypeMismatch(msg)
             return val
 
@@ -140,66 +146,79 @@ def deploy_time_eval(value):
     else:
         return value
 
+
 # this is called by cli.main
 def set_parameter_context(flow_name, echo, datastore):
     global context_proto
-    context_proto = ParameterContext(flow_name=flow_name,
-                                     user_name=get_username(),
-                                     parameter_name=None,
-                                     logger=echo,
-                                     ds_type=datastore.TYPE)
+    context_proto = ParameterContext(
+        flow_name=flow_name,
+        user_name=get_username(),
+        parameter_name=None,
+        logger=echo,
+        ds_type=datastore.TYPE,
+    )
+
 
 class Parameter(object):
     def __init__(self, name, **kwargs):
         self.name = name
         self.kwargs = kwargs
         # TODO: check that the type is one of the supported types
-        param_type = self.kwargs['type'] = self._get_type(kwargs)
+        param_type = self.kwargs["type"] = self._get_type(kwargs)
+        reserved_params = [
+            "params",
+            "with",
+            "max-num-splits",
+            "max-workers",
+            "tag",
+            "run-id-file",
+            "namespace",
+        ]
 
-        if self.name == 'params':
-            raise MetaflowException("Parameter name 'params' is a reserved "
-                                    "word. Please use a different "
-                                    "name for your parameter.")
+        if self.name in reserved_params:
+            raise MetaflowException(
+                "Parameter name '%s' is a reserved "
+                "word. Please use a different "
+                "name for your parameter." % (name)
+            )
 
         # make sure the user is not trying to pass a function in one of the
         # fields that don't support function-values yet
-        for field in ('show_default',
-                      'separator',
-                      'required'):
+        for field in ("show_default", "separator", "required"):
             if callable(kwargs.get(field)):
-                raise MetaflowException("Parameter *%s*: Field '%s' cannot "
-                                        "have a function as its value"\
-                                        % (name, field))
+                raise MetaflowException(
+                    "Parameter *%s*: Field '%s' cannot "
+                    "have a function as its value" % (name, field)
+                )
 
-        self.kwargs['show_default'] = self.kwargs.get('show_default', True)
+        self.kwargs["show_default"] = self.kwargs.get("show_default", True)
 
         # default can be defined as a function
-        default_field = self.kwargs.get('default')
+        default_field = self.kwargs.get("default")
         if callable(default_field) and not isinstance(default_field, DeployTimeField):
-            self.kwargs['default'] = DeployTimeField(name,
-                                                     param_type,
-                                                     'default',
-                                                     self.kwargs['default'],
-                                                     return_str=True)
+            self.kwargs["default"] = DeployTimeField(
+                name, param_type, "default", self.kwargs["default"], return_str=True
+            )
 
         # note that separator doesn't work with DeployTimeFields unless you
         # specify type=str
-        self.separator = self.kwargs.pop('separator', None)
+        self.separator = self.kwargs.pop("separator", None)
         if self.separator and not self.is_string_type:
-            raise MetaflowException("Parameter *%s*: Separator is only allowed "
-                                    "for string parameters." % name)
+            raise MetaflowException(
+                "Parameter *%s*: Separator is only allowed "
+                "for string parameters." % name
+            )
         parameters.append(self)
 
     def option_kwargs(self, deploy_mode):
         kwargs = self.kwargs
-        if isinstance(kwargs.get('default'), DeployTimeField) and not deploy_mode:
+        if isinstance(kwargs.get("default"), DeployTimeField) and not deploy_mode:
             ret = dict(kwargs)
-            help_msg = kwargs.get('help')
-            help_msg = '' if help_msg is None else help_msg
-            ret['help'] = help_msg + \
-                "[default: deploy-time value of '%s']" % self.name
-            ret['default'] = None
-            ret['required'] = False
+            help_msg = kwargs.get("help")
+            help_msg = "" if help_msg is None else help_msg
+            ret["help"] = help_msg + "[default: deploy-time value of '%s']" % self.name
+            ret["default"] = None
+            ret["required"] = False
             return ret
         else:
             return kwargs
@@ -210,22 +229,24 @@ class Parameter(object):
     def _get_type(self, kwargs):
         default_type = str
 
-        default = kwargs.get('default')
+        default = kwargs.get("default")
         if default is not None and not callable(default):
             default_type = type(default)
 
-        return kwargs.get('type', default_type)
+        return kwargs.get("type", default_type)
 
     @property
     def is_string_type(self):
-        return self.kwargs.get('type', str) == str and\
-               isinstance(self.kwargs.get('default', ''), strtype)
+        return self.kwargs.get("type", str) == str and isinstance(
+            self.kwargs.get("default", ""), strtype
+        )
 
     # this is needed to appease Pylint for JSONType'd parameters,
     # which may do self.param['foobar']
     def __getitem__(self, x):
         pass
 
+
 def add_custom_parameters(deploy_mode=False):
     # deploy_mode determines whether deploy-time functions should or should
     # not be evaluated for this command
@@ -234,26 +255,7 @@ def add_custom_parameters(deploy_mode=False):
         # in the order they are defined in the FlowSpec subclass
         for arg in parameters[::-1]:
             kwargs = arg.option_kwargs(deploy_mode)
-            cmd.params.insert(0, click.Option(('--' + arg.name,), **kwargs))
+            cmd.params.insert(0, click.Option(("--" + arg.name,), **kwargs))
         return cmd
+
     return wrapper
-
-def set_parameters(flow, kwargs):
-    seen = set()
-    for var, param in flow._get_parameters():
-        norm = param.name.lower()
-        if norm in seen:
-            raise MetaflowException("Parameter *%s* is specified twice. "
-                                    "Note that parameter names are "
-                                    "case-insensitive." % param.name)
-        seen.add(norm)
-
-    flow._success = True
-    for var, param in flow._get_parameters():
-        val = kwargs[param.name.replace('-', '_').lower()]
-        # Support for delayed evaluation of parameters. This is used for
-        # includefile in particular
-        if callable(val):
-            val = val()
-        val = val.split(param.separator) if val and param.separator else val
-        setattr(flow, var, val)
