@@ -29,17 +29,16 @@ try:
     def unquote_bytes(x):
         return to_unicode(unquote(to_bytes(x)))
 
-    # this is used e.g. by datastore.save_logs to identify paths
+    # this is used e.g. by mflog/save_logs.py to identify paths
     class Path(object):
-
-        def __init__(path):
+        def __init__(self, path):
             self.path = path
 
         def __str__(self):
             return self.path
 
     from pipes import quote as _quote
-except:
+except NameError:
     # python3
     unicode_type = str
     bytes_type = bytes
@@ -51,11 +50,13 @@ except:
 
     from shlex import quote as _quote
 
-if sys.version_info.major >= 3 and sys.version_info.minor >= 7:
+if sys.version_info >= (3, 7):
     from collections import namedtuple
+
     namedtuple_with_defaults = namedtuple
 else:
     from collections import namedtuple
+
     def namedtuple_with_defaults(typename, field_names, defaults=()):
         T = namedtuple(typename, field_names)
         T.__new__.__defaults__ = (None,) * len(T._fields)
@@ -77,10 +78,11 @@ class TempDir(object):
 def cached_property(getter):
     @wraps(getter)
     def exec_once(self):
-        saved_name = '__%s' % getter.__name__
+        saved_name = "__%s" % getter.__name__
         if not hasattr(self, saved_name):
             setattr(self, saved_name, getter(self))
         return getattr(self, saved_name)
+
     return property(exec_once)
 
 
@@ -110,7 +112,8 @@ def url_quote(url):
     #
     # We mark colon as a safe character to keep simple ASCII urls
     # nice looking, e.g. "http://google.com"
-    return to_bytes(quote(to_bytes(url), safe='/:'))
+    return to_bytes(quote(to_bytes(url), safe="/:"))
+
 
 def url_unquote(url_bytes):
     """
@@ -118,6 +121,7 @@ def url_unquote(url_bytes):
     """
     return unquote_bytes(url_bytes)
 
+
 def is_stringish(x):
     """
     Returns true if the object is a unicode or a bytes object
@@ -137,7 +141,7 @@ def to_unicode(x):
     Convert any object to a unicode object
     """
     if isinstance(x, bytes_type):
-        return x.decode('utf-8')
+        return x.decode("utf-8")
     else:
         return unicode_type(x)
 
@@ -147,13 +151,13 @@ def to_bytes(x):
     Convert any object to a byte string
     """
     if isinstance(x, unicode_type):
-        return x.encode('utf-8')
+        return x.encode("utf-8")
     elif isinstance(x, bytes_type):
         return x
     elif isinstance(x, float):
-        return repr(x).encode('utf-8')
+        return repr(x).encode("utf-8")
     else:
-        return str(x).encode('utf-8')
+        return str(x).encode("utf-8")
 
 
 def get_username():
@@ -162,33 +166,35 @@ def get_username():
     could not be determined.
     """
     # note: the order of the list matters
-    ENVVARS = ['METAFLOW_USER', 'SUDO_USER', 'USERNAME', 'USER']
+    ENVVARS = ["METAFLOW_USER", "SUDO_USER", "USERNAME", "USER"]
     for var in ENVVARS:
         user = os.environ.get(var)
-        if user and user != 'root':
+        if user and user != "root":
             return user
     return None
 
 
 def resolve_identity():
-    prod_token = os.environ.get('METAFLOW_PRODUCTION_TOKEN')
+    prod_token = os.environ.get("METAFLOW_PRODUCTION_TOKEN")
     if prod_token:
-        return 'production:%s' % prod_token
+        return "production:%s" % prod_token
     user = get_username()
-    if user and user != 'root':
-        return 'user:%s' % user
+    if user and user != "root":
+        return "user:%s" % user
     else:
         raise MetaflowUnknownUser()
 
 
 def get_latest_run_id(echo, flow_name):
-    from metaflow.datastore.local import LocalDataStore
-    local_root = LocalDataStore.datastore_root
+    from metaflow.datastore.local_storage import LocalStorage
+
+    local_root = LocalStorage.datastore_root
     if local_root is None:
-        v = LocalDataStore.get_datastore_root_from_config(echo, create_on_absent=False)
-        LocalDataStore.datastore_root = local_root = v
+        local_root = LocalStorage.get_datastore_root_from_config(
+            echo, create_on_absent=False
+        )
     if local_root:
-        path = os.path.join(local_root, flow_name, 'latest_run')
+        path = os.path.join(local_root, flow_name, "latest_run")
         if os.path.exists(path):
             with open(path) as f:
                 return f.read()
@@ -196,30 +202,33 @@ def get_latest_run_id(echo, flow_name):
 
 
 def write_latest_run_id(obj, run_id):
-    from metaflow.datastore.local import LocalDataStore
-    if LocalDataStore.datastore_root is None:
-        LocalDataStore.datastore_root = LocalDataStore.get_datastore_root_from_config(obj.echo)
-    path = os.path.join(LocalDataStore.datastore_root, obj.flow.name)
+    from metaflow.datastore.local_storage import LocalStorage
+
+    if LocalStorage.datastore_root is None:
+        LocalStorage.datastore_root = LocalStorage.get_datastore_root_from_config(
+            obj.echo
+        )
+    path = LocalStorage.path_join(LocalStorage.datastore_root, obj.flow.name)
     try:
         os.makedirs(path)
     except OSError as x:
         if x.errno != 17:
-            # Directories exists in other casewhich is fine
+            # Directories exists in other case which is fine
             raise
-    with open(os.path.join(path, 'latest_run'), 'w') as f:
+    with open(os.path.join(path, "latest_run"), "w") as f:
         f.write(str(run_id))
 
 
 def get_object_package_version(obj):
-    """    
+    """
     Return the top level package name and package version that defines the
     class of the given object.
     """
     try:
         module_name = obj.__class__.__module__
 
-        if '.' in module_name:
-            top_package_name = module_name.split('.')[0]
+        if "." in module_name:
+            top_package_name = module_name.split(".")[0]
         else:
             top_package_name = module_name
 
@@ -234,17 +243,14 @@ def get_object_package_version(obj):
         return top_package_name, None
 
 
-def compress_list(lst,
-                  separator=',',
-                  rangedelim=':',
-                  zlibmarker='!',
-                  zlibmin=500):
+def compress_list(lst, separator=",", rangedelim=":", zlibmarker="!", zlibmin=500):
 
-    bad_items = [x for x in lst
-                 if separator in x or rangedelim in x or zlibmarker in x]
+    bad_items = [x for x in lst if separator in x or rangedelim in x or zlibmarker in x]
     if bad_items:
-        raise MetaflowInternalError("Item '%s' includes a delimiter character "
-                                    "so it can't be compressed" % bad_items[0])
+        raise MetaflowInternalError(
+            "Item '%s' includes a delimiter character "
+            "so it can't be compressed" % bad_items[0]
+        )
     # Three output modes:
     lcp = longest_common_prefix(lst)
     if len(lst) < 2 or not lcp:
@@ -264,14 +270,15 @@ def compress_list(lst,
         # has plenty of redundancy. Decoding the data *twice* helps a
         # lot
         compressed = zlib.compress(zlib.compress(to_bytes(res)))
-        return zlibmarker + base64.b64encode(compressed).decode('utf-8')
+        return zlibmarker + base64.b64encode(compressed).decode("utf-8")
 
-def decompress_list(lststr, separator=',', rangedelim=':', zlibmarker='!'):
+
+def decompress_list(lststr, separator=",", rangedelim=":", zlibmarker="!"):
     # Three input modes:
     if lststr[0] == zlibmarker:
         # 3. zlib-compressed, base64-encoded
         lstbytes = base64.b64decode(lststr[1:])
-        decoded = zlib.decompress(zlib.decompress(lstbytes)).decode('utf-8')
+        decoded = zlib.decompress(zlib.decompress(lstbytes)).decode("utf-8")
     else:
         decoded = lststr
 
@@ -286,15 +293,17 @@ def decompress_list(lststr, separator=',', rangedelim=':', zlibmarker='!'):
 
 def longest_common_prefix(lst):
     if lst:
-        return ''.join(a for a, _ in takewhile(lambda t: t[0] == t[1],
-                                               zip(min(lst), max(lst))))
+        return "".join(
+            a for a, _ in takewhile(lambda t: t[0] == t[1], zip(min(lst), max(lst)))
+        )
     else:
-        return ''
+        return ""
 
 
 def get_metaflow_root():
     return os.path.dirname(os.path.dirname(__file__))
 
+
 def dict_to_cli_options(params):
     for k, v in params.items():
         # Omit boolean options set to false or None, but preserve options with an empty
@@ -302,19 +311,19 @@ def dict_to_cli_options(params):
         if v is not False and v is not None:
             # we need special handling for 'with' since it is a reserved
             # keyword in Python, so we call it 'decospecs' in click args
-            if k == 'decospecs':
-                k = 'with'
-            k = k.replace('_', '-')
+            if k == "decospecs":
+                k = "with"
+            k = k.replace("_", "-")
             if not isinstance(v, tuple):
                 v = [v]
             for value in v:
-                yield '--%s' % k
+                yield "--%s" % k
                 if not isinstance(value, bool):
                     value = to_unicode(value)
 
                     # Of the value starts with $, assume the caller wants shell variable
                     # expansion to happen, so we pass it as is.
-                    # NOTE: We strip '\' to allow for various backends to use escaped
+                    # NOTE: We strip '\' to allow for various storages to use escaped
                     # shell variables as well.
                     if value.lstrip("\\").startswith("$"):
                         yield value
@@ -338,8 +347,10 @@ def which(cmd, mode=os.F_OK | os.X_OK, path=None):
     # directories pass the os.access check.
     try:  # Forced testing
         from shutil import which as w
+
         return w(cmd, mode, path)
-    except ImportError: 
+    except ImportError:
+
         def _access_check(fn, mode):
             return os.path.exists(fn) and os.access(fn, mode) and not os.path.isdir(fn)
 
@@ -381,9 +392,9 @@ def to_pascalcase(obj):
     if isinstance(obj, dict):
         res = obj.__class__()
         for k in obj:
-            res[re.sub('([a-zA-Z])', 
-                lambda x: x.groups()[0].upper(), k, 1)] = \
-                to_pascalcase(obj[k])
+            res[
+                re.sub("([a-zA-Z])", lambda x: x.groups()[0].upper(), k, 1)
+            ] = to_pascalcase(obj[k])
     elif isinstance(obj, (list, set, tuple)):
         res = obj.__class__(to_pascalcase(v) for v in obj)
     else:
